@@ -1,7 +1,7 @@
 /*
  * Soft:        multipath device mapper target autoconfig
  *
- * Version:     $Id: main.c,v 1.26 2005/07/20 19:44:54 bmarzins Exp $
+ * Version:     $Id: main.c,v 1.27 2005/07/31 20:38:18 bmarzins Exp $
  *
  * Author:      Copyright (C) 2003 Christophe Varoqui
  *
@@ -525,6 +525,18 @@ pgcmp (struct multipath * mpp, struct multipath * cmpp)
 	return r;
 }
 
+static int
+num_paths (struct multipath * mpp)
+{
+	struct pathgroup *pgp;
+	int i;
+	int count = 0;
+
+	vector_foreach_slot (mpp->pg, pgp, i)
+		count += VECTOR_SIZE(pgp->paths);
+	return count;
+}
+
 static void
 select_action (struct multipath * mpp, vector curmp)
 {
@@ -533,6 +545,11 @@ select_action (struct multipath * mpp, vector curmp)
 	cmpp = find_mp(curmp, mpp->alias);
 
 	if (!cmpp) {
+		if (conf->need_multiple && num_paths(mpp) < 2){
+			condlog(3, "only one path, doing nothing");
+			mpp->action = ACT_NOTHING;
+			return;
+		}
 		mpp->action = ACT_CREATE;
 		return;
 	}
@@ -771,6 +788,7 @@ usage (char * progname)
 		"\t-ll\t\tshow multipath topology (maximum info)\n" \
 		"\t-F\t\tflush a multipath device map\n" \
 		"\t-F\t\tflush all multipath device maps\n" \
+		"\t-m\t\tonly setup map if it has multiple paths\n" \
 		"\t-p policy\tforce all maps to specified policy :\n" \
 		"\t   failover\t\t1 path per priority group\n" \
 		"\t   multibus\t\tall paths in 1 priority group\n" \
@@ -876,7 +894,7 @@ main (int argc, char *argv[])
 	if (load_config(DEFAULT_CONFIGFILE))
 		exit(1);
 
-	while ((arg = getopt(argc, argv, ":qdl::Ffi:M:v:p:")) != EOF ) {
+	while ((arg = getopt(argc, argv, ":qdl::Ffi:M:mv:p:")) != EOF ) {
 		switch(arg) {
 		case 1: printf("optarg : %s\n",optarg);
 			break;
@@ -909,6 +927,9 @@ main (int argc, char *argv[])
 			debug = atoi(optarg);
 #endif
 			break;
+		case 'm':
+			conf->need_multiple = 1;
+			break;
 		case 'p':
 			conf->pgpolicy_flag = get_pgpolicy_id(optarg);
 			if (conf->pgpolicy_flag == -1) {
