@@ -1,7 +1,7 @@
 /*
  * Soft:        Description here...
  *
- * Version:     $Id: main.h,v 1.17 2005/06/08 19:29:41 agk Exp $
+ * Version:     $Id: main.h,v 1.18 2005/07/20 19:44:54 bmarzins Exp $
  *
  * Author:      Copyright (C) 2003 Christophe Varoqui
  *
@@ -35,6 +35,9 @@ enum actions {
 	ACT_CREATE
 };
 
+#define FLUSH_ONE 1
+#define FLUSH_ALL 2
+
 /*
  * Build version
  */
