@@ -205,6 +205,13 @@ setup_multipath (struct paths * allpaths, struct multipath * mpp)
 {
 	char * wwid;
 	int i;
+	int r = 1;
+
+	if (!dm_map_present(mpp->alias)) {
+		condlog(2, "dm map %s removed", mpp->alias);
+		r = 2;
+		goto out;
+	}
 
 	wwid = get_mpe_wwid(mpp->alias);
 
@@ -232,8 +239,9 @@ out:
 		vector_del_slot(allpaths->mpvec, i);
 
 	free_multipath(mpp, KEEP_PATHS);
-	condlog(0, "failed to setup multipath");
-	return 1;
+	if (r == 1)
+		condlog(2, "failed to setup multipath");
+	return r;
 }
 
 static void
@@ -279,7 +287,8 @@ update_multipath (struct paths *allpaths, char *mapname)
 	free_pgvec(mpp->pg, KEEP_PATHS);
 	mpp->pg = NULL;
 
-	if (setup_multipath(allpaths, mpp))
+	r = setup_multipath(allpaths, mpp);
+	if (r)
 		goto out; /* mpp freed in setup_multipath */
 
 	/*
@@ -303,10 +312,9 @@ update_multipath (struct paths *allpaths, char *mapname)
 			}
 		}
 	}
-	r = 0;
 out:
-	if (r)
-		condlog(0, "failed to update multipath");
+	if (r == 1)
+		condlog(2, "failed to update multipath");
 
 	return r;
 }
